class RequestExceptionsBuilder(Exception):

    def __init__(self, message: str, response_code: int, uri: str, errors: list = None):
        """
        :param str message:
        :param int response_code:
        :param str uri:
        :param errors:
        :return Exception
        """

        self.method = uri
        self.errors = errors if errors else []
        super(RequestExceptionsBuilder, self).__init__(message, response_code)

    def get_method(self):
        return self.method

    def get_errors(self):
        return self.errors



from .payments import Payment
from .payouts import Payout


class Client:

    def __init__(self):
        pass

    @staticmethod
    def payment(payment_key: str, merchant_uuid: str) -> Payment:
        """
        :param str payment_key: Api Key from merchant settings
        :param str merchant_uuid: Merchant ID
        :return class Payment
        """

        return Payment(payment_key, merchant_uuid)

    @staticmethod
    def payout(payout_key: str, merchant_uuid: str) -> Payout:
        """
        :param str payout_key:
        :param str merchant_uuid:
        :return Payout:
        """

        return Payout(payout_key, merchant_uuid)



import requests
import json
import hashlib
import base64
from .request_exceptions import RequestExceptionsBuilder


class RequestBuilder:

    def __init__(self, secret_key: str, merchant_uuid: str):
        """
        :param str secret_key:
        :param str merchant_uuid:
        :param
        """
        self.api_url = 'https://api.heleket.com/'
        self.secret_key = secret_key
        self.merchant_uuid = merchant_uuid

    def send_request(self, uri: str, data: dict):
        """
        :param str uri: uri is the piece of link added to main Api Url to make strict request
        :param dict data: body data of request
        :return JSON result:
        """

        url = self.api_url + uri

        json_body_data = json.dumps(data, separators=(',', ':'))
        json_body_data_binary = json_body_data.encode('utf-8')

        encoded_data = base64.b64encode(json_body_data_binary)
        sign_md5_obj = hashlib.md5(encoded_data + self.secret_key.encode('utf-8'))
        sign = sign_md5_obj.hexdigest()

        headers = {
            "Accept": "application/json",
            "Content-Type": "application/json;charset=UTF-8",
            "Content-Legth": str(len(json_body_data)),
            "merchant": self.merchant_uuid,
            "sign": sign
        }

        try:
            response = requests.post(url=url, headers=headers, data=json_body_data)
        except Exception as e:
            raise RequestExceptionsBuilder('Problems with requests library', response_code=0, uri=uri, errors=[e])

        response_code = response.status_code

        try:
            response_json = response.json()
        except Exception as e:
            raise RequestExceptionsBuilder(str(e), response_code=response_code, uri=uri)

        if response_json:
            if response_code != 200 or (response_json['state'] is not None and response_json['state'] != 0):
                if 'message' in response_json:
                    raise RequestExceptionsBuilder(response_json['message'], response_code, uri)
                if 'errors' in response_json:
                    raise RequestExceptionsBuilder('Validation error', response_code, uri,
                                                   errors=response_json['errors'])

            if 'result' in response_json and response_json['state'] is not None and response_json['state'] == 0:
                return response_json['result']

        return True



from .request_builder import RequestBuilder


class Payment:

    def __init__(self, payment_key: str, merchant_uuid: str):
        self.payment_key = payment_key
        self.merchant_uuid = merchant_uuid
        self.version = 'v1'
        self.requestBuilder = RequestBuilder(payment_key, merchant_uuid)

    def create(self, data: dict):
        """
        :param dict data:
                         str amount: Amount to pay
                         str currency: Payment currency
                         str network: Payment network
                         str order_id: Order ID in your system
                         str url_return: Redirect link
                         str url_callback: Callback link
                         bool is_payment_multiple: Allow surcharges on payment *
                         str lifetime: Payment lifetime in seconds
                         str to_currency: Currency to convert amount to
        :return Optional(bool):
        :raise RequestBuilderException
        """

        uri = self.version + '/payment'
        return self.requestBuilder.send_request(uri, data)

    def info(self, data: dict):
        """
        :param data:
                    str uuid: Transaction uuid
                    str order_id: Order id
        :return : request answer in JSON format
        """

        uri = self.version + '/payment/info'
        return self.requestBuilder.send_request(uri, data)

    def history(self, page: int = 1, **parameters):
        """
        :param int page: Pagination cursor
        :param dict parameters: Additional parameters
        :return json : request answer in JSON format
        :raise RequestBuilderException
        """
        if parameters is None:
            parameters = {}

        uri = self.version + '/payment/list'
        parameters['cursor'] = str(page)

        return self.requestBuilder.send_request(uri, data=parameters)

    def services(self, **parameters):
        """
        :param dict parameters: Additional parameters
        :return request answer in JSON format
        :raise RequestBuilderException
        """
        if parameters is None:
            paramenters = {}

        uri = self.version + '/payment/services'
        return self.requestBuilder.send_request(uri, parameters)

    def balance(self, **parameters):
        """
        :param parameters:
        :return json balance:
        :raise RequestBuilderException
        """
        if parameters is None:
            parameters = {}

        uri = self.version + '/balance'
        return self.requestBuilder.send_request(uri, parameters)

    def resend_notification(self, data: dict):
        """
        :param dict data:
                        str uuid: Payment's UUID
                        str order_id: Order ID in your system
        :return Optional bool
        :raise RequestBuilderException
        """
        uri = self.version + '/payment/resend'
        return self.requestBuilder.send_request(uri, data)

    def create_wallet(self, data: dict):
        """
        :param dict data:
                        str network: Network
                        str currency: Payment currency
                        str order_id: Order ID in your system
                        str url_callback: Callback url
        :return Optional(bool): returns Bool or mixed data
        :raise RequestBuilderException
        """
        uri = self.version + '/wallet'
        return self.requestBuilder.send_request(uri, data)










from setuptools import setup

long_desc = open('README.md').read()
print(long_desc)

setup(name='heleket',
      version='1.1',
      description='Python SDK for Heleket.com API',
      long_description_content_type='text/markdown',
      long_description=long_desc,
      packages=['heleket'],
      keywords=['Heleket', 'Heleket.com', 'API'],
      author='Daniil Gord',
      author_email='lexicon-nothing.0y@icloud.com',
      zip_safe=False,
      install_requires=['requests']
      )



from .request_builder import RequestBuilder


class Payout:

    def __init__(self, payout_key: str, merchant_uuid: str):
        self.payment_key = payout_key
        self.merchant_uuid = merchant_uuid
        self.version = 'v1'
        self.request_builder = RequestBuilder(payout_key, merchant_uuid)

    def create(self, data):
        """
        :param dict data:
                        str amount: Amount to payout
                        str currency: Currency
                        str network: Network
                        str order_id: Your id
                        str address: Wallet number
                        str is_subtract: From where to withdraw the commission (1 - from the balance, 0 - from the amount)
                        str url_callback: Callback link
        :return Optional(bool):
        :raise RequestBuilderException
        """

        uri = self.version + '/payout'
        return self.request_builder.send_request(uri, data)

    def info(self, data):
        """
        :param data:
                    str uuid
        :return Optional(bool):
        :raise RequestBuilderException
        """

        uri = self.version + '/payout/info'
        return self.request_builder.send_request(uri, data)
